%gtPreprocessing - "do all" script to handle all preprocessing
%parameters file, created by gtSetup, should have all required directorry
%info.  Run this script in the top level analysis directory (containig the
%parameters file)
%live! launch a series of condor jobs which will sit and wait
%gtCopyCorrectUndistortWrapper launches a number of instances of
%condor_... according to how many files are present in the directory
%when the function is started.

function gtPreprocessing_live


freshcompile=false;  % recompile binaries - not often needed.  But more often than it should be...
if freshcompile
  disp('recompiling all functions')
  id19_mcc('sequence_median_refs')
  gt_mcc('create_abs_live');
  gt_mcc('gtAbsMedian_live');
  gt_mcc('gtMovingMedian_live');
  gt_mcc('create_full_live');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load('parameters.mat');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if exist('parameters.mat', 'file')
    load('parameters.mat');;
else
    disp('first run gtSetup to create parameters file for the experiment, or go to the directory containing this file!')
    return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% xml file exists?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
xmlfname=sprintf('0_rawdata/%s/%s.xml',parameters.acq.name,parameters.acq.name);
if ~exist(xmlfname,'file')
    disp('No .xml file found in the analysis directory!  quitting...')
    disp('gt_sequence_median_refs should be modified to work from parameters OR xml')
    return
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% set parameters file correct for start of preprocessing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% after the copying has been done, the collection_dir in parameters
% points to an empty directory.  Change so collection_dir points to the
% /0_rawdata/Orig folder, and save the original collection_dir name
if ~isfield(parameters.acq, 'collection_dir_old')
    parameters.acq.collection_dir_old=parameters.acq.collection_dir;
    parameters.acq.collection_dir=[parameters.acq.dir '/0_rawdata/Orig'];
end

% when we launch preprocessing, must reset renumbered to false, as this
% step must be done after creating the images
if parameters.acq.interlaced_turns>0
    parameters.prep.renumbered=false;
end

save parameters parameters

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% how many images in this scan?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if strcmp(acq.type, '360degree')
    totproj=2*acq.nproj;
elseif strcmp(acq.type, '180degree')
    totproj=acq.nproj;
else
    Mexc = MException('ACQ:invalid_parameter', ...
                      'Unknown type of scan. Check parameters.acq.type!');
    throw(Mexc);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% create dark.edf file of scan
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('Creating dark.edf image of scan.')
dark=edf_read(sprintf('0_rawdata/%s/darkend0000.edf', parameters.acq.name))/parameters.acq.ndark;
edf_write(dark,sprintf('0_rawdata/%s/dark.edf', parameters.acq.name),'float32');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% acq/prep parameters determined from images
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% if processing "live", may not have all images required.  Use sensible
% defaults or compromises, that can be refined later, and give warnings!

check='y';
if isfield(parameters.acq, 'rotu') && isfield(parameters.acq, 'bb')
    disp('sample bounding box and centre of rotation already exist!')
    check=inputwdefault('Recalculate these parameters? [y/n]', 'y');
end

if strcmpi(check,'y')
    % find direct beam bounding box
    bbdir=gtFindDirectBeamBB(parameters.acq);
    % determine rotation axis
    % "u" is the horizontal pixel coordinate
    parameters.acq.rotu=gtFindRotationAxis(parameters.acq,bbdir);
    %determine the limits of sample horizontally for acq.bb
    %this should be centred on the rotation axis
    parameters.acq.bb=gtFindSampleEdges(parameters.acq,parameters.prep,bbdir);
    % save these values in parameters
    save parameters parameters
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate sample drifts
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% this is the time to analyse a calibration or quali scan to determine
% sample drifts
check=inputwdefault('Is there a calibration or quali scan for this measurement? [y/n]', 'n');
if strcmpi(check,'y')
    disp('Quali scan analysis')
    if isfield(parameters, 'calib') && isfield(parameters.calib, 'zdrift') && isfield(parameters.calib, 'ydrift')
        disp('parameters file already has calibration scan data')
        check=inputwdefault('Repeat calibration scan analysis? [y/n]', 'y');
        if strcmpi(check, 'y')
            showcorr=inputwdefault('show image  correlations? [y/n]', 'y');
            if strcmpi(showcorr, 'y')
                showcorr=true;
            else
                showcorr=false;
            end
            disp('calling gtSetupCalibrationQuali.m')
            warning('* caution * not sure if coordinate system changes or non-centred direct beam bb could affect this')
            parameters = gtSetupCalibrationQuali(bbdir,showcorr);
            % output is automatically saved into parameters file
            % as parameters.calib.ydrift and parameters.calib.zdrift
            % modified function to also return updated parameters
        end
    end
    ydrift=parameters.calib.ydrift;
    zdrift=parameters.calib.zdrift;
    % can we also get a better rotu value out of the calibration scan
    % analysis?
else
    % do not apply drifts
    disp('Not applying any drift correction')
    ydrift=zeros(1, totproj);
    zdrift=zeros(1, totproj);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% shift images:
% rotation axis to centre, plus apply drifts
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% combine sample drifts and centre of rotation correction
parameters.prep.udrift=ydrift + (parameters.acq.xdet/2)+0.5 - parameters.acq.rotu;
parameters.prep.vdrift=zdrift;
save parameters parameters

if all(parameters.prep.udrift==0) && all(parameters.prep.vdrift==0)
    parameters.prep.correct_drift='not_required';
else
    parameters.prep.correct_drift='required';
    save parameters parameters
    disp('setting up a database table for gtApplyDrifts')
    gtDBConnect
    drifttable=[parameters.acq.name '_drifttable'];
    mym(sprintf('drop table if exists %s',drifttable));
    mym(sprintf('create table if not exists %s (filename text, unique key (filename(40)))', drifttable))
    % launch condor jobs
    condor_make('gtApplyDrifts', 1, 100, 15, parameters.acq.dir, 1)
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% launch preprocessing on condor
% all functions use a modified pfWaitToRead that waits for images to be
% shifted by gtApplyDrifts (if required)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% gtSequenceMedianRefs : make refHST in raw data
condor_make('gtSequenceMedianRefs', 0, 0, 1, [parameters.acq.dir ' overwrite'], true, use_cronii)
disp('gtSequenceMedianRefs launched')

% gtCreateAbsLive
condor_make('gtCreateAbsLive', 0, totproj, 1, parameters.acq.dir, true, use_cronii);
disp('gtCreateAbsLive launched')

% gtAbsMedianLive
condor_make('gtAbsMedianLive', 0, totproj, 1, parameters.acq.dir, true, use_cronii);
disp('gtAbsMedianLive launched')

% gtMovingMedianLive
condor_make('gtMovingMedianLive', 0, totproj, 1, parameters.acq.dir, true, use_cronii);
disp('gtMovingMedianLive launched')

% gtCreateFullLive
condor_make('gtCreateFullLive', 0, totproj, 30, parameters.acq.dir, true, use_cronii);
disp('gtCreateFullLive launched')





