function gtShowFsim(grain, phaseid, clims)
% GTSHOWFSIM Shows the forward simulation result
%   Usage:
%       gtShowFsim(grainid, phaseid, clims)
%       gtShowFsim(grain_structure, phaseid, clims)
%           (in this case, phase ID is ignored)



    if isnumeric(grain)
        if ~exist('phaseid', 'var')
            phaseid = 1;
        end
        filename = sprintf('4_grains/phase_%02d/grain_%04d.mat', phaseid, grain);
        grain = load(filename);
    end

    grainid = grain.id;
    phaseid = grain.phaseid;
    gtDBConnect();

    hasConflictInfo = exist('4_grains/grains_conflicts.mat', 'file');
    if (hasConflictInfo)
        grains_conflicts = [];
        load('4_grains/grains_conflicts.mat');

        idspots = grains_conflicts{phaseid}.getIdSpotRow(grainid,:);
        fprintf('Found %d spots in common with other grain(s)\n',numel(idspots))
        disp(idspots)

        numSpots = numel(idspots);
        id = zeros(numSpots, 1);
        x = zeros(numSpots, 1);
        y = zeros(numSpots, 1);
        im = zeros(numSpots, 1);

        for ii = 1:numSpots
            % XXX - Add handling of correct dataset name!!
            query = sprintf(...
                [ 'SELECT difspotID, CentroidX, CentroidY, CentroidImage ' ...
                  '     FROM RD4_15N_difspot' ...
                  '     WHERE difspotID = %d' ], ...
                idspots(ii));
            [id(ii), x(ii), y(ii), im(ii)] = mym(query);
        end

        idconflicts = grains_conflicts{phaseid}.getIdConflict(grainid);
        for kk = 1:numel(idconflicts)
            [~, indexes] = intersect(grain.difspotID, idspots, 'stable');
            indexes = indexes';
            fprintf('  idconflict : %d   indexes: ', idconflicts(kk))
            fprintf('%d ', indexes)
            disp(' ')
            flags = grain.flag';
            selected = find(flags == 4 | flags == 5);
            disp('  Original flags for difspots:')
            disp(flags(selected(indexes)))
            flags(selected(indexes)) = 6;
            grain.oldflag = grain.flag;
            grain.flag = flags;
            grain.conflicts = selected(indexes);
        end
        % on going ....
    else
        disp('No difspots in conflict found')
    end

    if (~exist('clims', 'var'))
        clims = 0;
    end

    %leg = { 'flag = 1 : No intensity in raw images', ...
    %        'flag = 2 : Found some intensity in raw images', ...
    %        'flag = 3 : Found segmented intensity - not matching search criteria', ...
    %        'flag = 4 : Found difspot matching search criteria', ...
    %        'flag = 5 : Difspot (pair) identified by indexter', ...
    %        'flag = 6 : Spot claimed by another grain' };

    leg = { ...
        '1: No intensity in raw images', ...
        '2: Intensity in raw images', ...
        sprintf('3: Segmented intensity not\n    matching search criteria'), ...
        '4: Difspot matching search criteria', ...
        '5: Indexed difspot as Friedel pair', ...
        '6: Spot claimed by another grain'};

    clims = gtSetColourLimits(grain.full, clims);

    ondet = grain.ondet;

    f = figure('Units','normalized','Position',[0 0 0.8 1]);
    %set(f,);
    ax = axes('Parent', f);

    colormap(ax, gray);
    imagesc(grain.full, 'Parent', ax, clims);
    axis(ax, 'equal');
    axis(ax, [1 size(grain.full, 2) 1 size(grain.full, 1)])

    gtBrightnessContrast(ax);
    hold(ax, 'on');

    title(sprintf('Phase %d - Grain %d', phaseid, grainid));
    % flag = 1  no intensity in raw images...!
    ind   = find(grain.flag == 1);
    u_vec = grain.allblobs.uv(ondet(ind), 1);
    v_vec = grain.allblobs.uv(ondet(ind), 2);
    if ~isempty(ind)
        h(1) = plot(u_vec, v_vec,'rx');
    else
        h(1) = plot(1, -10 ,'rx');
    end

    % flag = 2 some segmented intensity found at predicted spot position
    ind   = find(grain.flag == 2);
    u_vec = grain.allblobs.uv(ondet(ind), 1);
    v_vec = grain.allblobs.uv(ondet(ind), 2);
    if ~isempty(ind)
        h(2) = plot(u_vec,v_vec,'mo');
    else
        h(2) = plot(1, -10,'mo');
    end

    % flag = 3 some segmented intensity found at predicted spot position
    ind   = find(grain.flag == 3);
    u_vec = grain.allblobs.uv(ondet(ind), 1);
    v_vec = grain.allblobs.uv(ondet(ind), 2);
    if ~isempty(ind)
        h(3) = plot(u_vec,v_vec,'bo');
    else
        h(3) = plot(1, -10, 'bo');
    end

    % flag = 4  new valid spot collected during forward simulation
    ind   = find(grain.flag == 4);
    u_vec = grain.allblobs.uv(ondet(ind), 1);
    v_vec = grain.allblobs.uv(ondet(ind), 2);
    if ~isempty(ind)
        h(4) = plot(u_vec,v_vec,'gd');
    else
        h(4) = plot(1, -10, 'gd');
    end

    % flag = 5  spot has been identified by INDEXTER
    ind   = find(grain.flag == 5);
    u_vec = grain.allblobs.uv(ondet(ind), 1);
    v_vec = grain.allblobs.uv(ondet(ind), 2);
    if ~isempty(ind)
        h(5) = plot(u_vec,v_vec,'gh');
    else
        h(5) = plot(1, -10, 'gh');
    end

    if (hasConflictInfo)
        % flag = 6  spot claimed by another grain
        ind   = find(grain.flag == 6);
        u_vec = grain.allblobs.uv(ondet(ind), 1);
        v_vec = grain.allblobs.uv(ondet(ind), 2);
        if ~isempty(ind)
            h(6) = plot(u_vec,v_vec,'yo', 'MarkerSize', 20);
        else
            h(6) = plot(1, -10, 'yo', 'MarkerSize', 20);
        end

        % Only in case we have it
        text(x,y,num2str(id'),'Color','y')
    end
  
    % flag = 0  no spot search - just show expected positions
    ind = find(grain.flag == 0);
    if ~isempty(ind == length(grain.flag))
        u_vec = grain.allblobs.uv(ondet(ind), 1);
        v_vec = grain.allblobs.uv(ondet(ind), 2);
        h = plot(u_vec,v_vec,'bo');
        leg = 'expected spot positions';
    end

    %legend(h, leg, 'Location', 'NorthOutside')
    hl = legend(h, leg, 'Location', 'EastOutside');

    % Position figure, axes and legend
    set(f,'Units','normalized')
    set(f,'Position',[0 0 0.8 1])

    set(hl,'Units','normalized')
    pos = get(hl,'Position');
    set(hl,'Position',[0.99-pos(3) pos(2) pos(3) pos(4)])

    set(ax,'Units','normalized')
    pos = get(ax,'Position');
    set(ax,'Position',[pos(1) 0.05 pos(3) 0.9])

    set(f,'Units','normalized')
    set(f,'Position',[0 0 0.8 1])
end
