%
% Gives the closest point to any number of 3D lines given; `point of intersection` in 3D
% space (using explicit solution of least squares method; HIGHEST PERFORMANCE POSSIBLE)
%
% INPUT:  l(n,6) [ point_X point_Y point_Z norm_dir_X norm_dir_Y norm_dir_Z] =
%           any number of 3D lines in matrix form, direction vector normalized
%             
% OUTPUT: out[ X;Y;Z ] = `point of intersection` in 3D space
%   

function out = pofintersectexp(l)

    A(1,1) = sum(l(:,6).^2 + l(:,5).^2) ;    %    A(1,1) + v(3)^2+v(2)^2 ;
    A(1,2) = sum(-l(:,5).*l(:,4)) ;        %    A(1,2) - v(2)*v(1) ;
    A(1,3) = sum(-l(:,6).*l(:,4)) ;         %    A(1,3) - v(3)*v(1) ;

    A(2,1) = sum(-l(:,5).*l(:,4)) ;           %    A(2,1) - v(2)*v(1) ;
    A(2,2) = sum(l(:,6).^2 + l(:,4).^2) ;       %    A(2,2) + v(3)^2+v(1)^2 ;
    A(2,3) = sum(-l(:,6).*l(:,5)) ;          %    A(2,3) - v(3)*v(2) ;

    A(3,1) = sum(-l(:,4).*l(:,6)) ;           %    A(3,1) - v(1)*v(3) ;
    A(3,2) = sum(-l(:,5).*l(:,6)) ;            %    A(3,2) - v(2)*v(3) ;
    A(3,3) = sum(l(:,4).^2 + l(:,5).^2) ;       %    A(3,3) + v(1)^2+v(2)^2 ;

    b(1,1) = sum( (l(:,6).^2+l(:,5).^2).*l(:,1) - l(:,4).*(l(:,6).*l(:,3)+l(:,5).*l(:,2)) ) ;  %  v(3)^2*p(1) + v(2)^2*p(1) - v(3)*v(1)*p(3) - v(2)*v(1)*p(2) ;
    b(2,1) = sum( (l(:,6).^2+l(:,4).^2).*l(:,2) - l(:,5).*(l(:,6).*l(:,3)+l(:,4).*l(:,1)) ) ;  %  v(3)^2*p(2) + v(1)^2*p(2) - v(3)*v(2)*p(3) - v(2)*v(1)*p(1) ;
    b(3,1) = sum( (l(:,5).^2+l(:,4).^2).*l(:,3) - l(:,6).*(l(:,5).*l(:,2)+l(:,4).*l(:,1)) ) ;  %  v(2)^2*p(3) + v(1)^2*p(3) - v(3)*v(2)*p(2) - v(1)*v(3)*p(1) ;

out=A\b ;