classdef GtGuiAssembleVol3D < GtBaseGuiElem
    properties
        defaultConf;

        assembler;
    end
    methods (Access = public)
        function obj = GtGuiAssembleVol3D(varargin)
            obj = obj@GtBaseGuiElem();
            obj.defaultConf = GtAssembleVol3D.generateDefaults();

            obj.assembler = GtAssembleVol3D();
            guiArguments = obj.assembler.changeParams(varargin);

            obj.conf.f_title = 'Assemble GUI';
            obj.initGtBaseGuiElem(guiArguments);
        end
    end

    methods (Access = protected)
        function initGui(obj)
            initGui@GtBaseGuiElem(obj)

            asm_boxes = [];
            asm_boxes.main = uiextras.VBox('Parent', obj.conf.currentParent);
            asm_boxes.param_boxes = uiextras.VBox('Parent', asm_boxes.main);
            asm_boxes.butt_a_boxes = uiextras.HButtonBox('Parent', asm_boxes.main);
            asm_boxes.butt_d_boxes = uiextras.HButtonBox('Parent', asm_boxes.main);

            set(asm_boxes.main, 'Sizes', [-1 40 40]);

            buttonSize = get(asm_boxes.butt_a_boxes, 'ButtonSize');
            buttonSize(1) = 140;
            set(asm_boxes.butt_a_boxes, 'ButtonSize', buttonSize);
            set(asm_boxes.butt_d_boxes, 'ButtonSize', buttonSize);

            % Conf choice
            asm_boxes = obj.createConfChoices(asm_boxes);

            % Buttons
            asm_boxes.butts = [];
            asm_boxes.butts.butt_phase = uicontrol('Parent', asm_boxes.butt_a_boxes, ...
                                                'Style', 'pushbutton', ...
                                                'String', 'Assemble Phase', ...
                                                'Callback', @(src, evt)callbackButtSinglePhase(obj));
            asm_boxes.butts.edit_phase = uicontrol('Parent', asm_boxes.butt_a_boxes, ...
                                                'Style', 'edit', ...
                                                'String', '1', ...
                                                'Callback', @(src, evt)callbackButtSinglePhase(obj));
            asm_boxes.butts.butt_phase = uicontrol('Parent', asm_boxes.butt_a_boxes, ...
                                                'Style', 'pushbutton', ...
                                                'String', 'Assemble All Phases', ...
                                                'Callback', @(src, evt)callbackButtAllPhases(obj));
            asm_boxes.butts.butt_vol = uicontrol('Parent', asm_boxes.butt_a_boxes, ...
                                                'Style', 'pushbutton', ...
                                                'String', 'Assemble Volume', ...
                                                'Callback', @(src, evt)callbackButtVolume(obj));
            asm_boxes.butts.butt_reset = uicontrol('Parent', asm_boxes.butt_a_boxes, ...
                                                'Style', 'pushbutton', ...
                                                'String', 'Reset', ...
                                                'Callback', @(src, evt)callbackButtReset(obj));

            asm_boxes.butts.butt_dilate = uicontrol('Parent', asm_boxes.butt_d_boxes, ...
                                                'Style', 'pushbutton', ...
                                                'String', 'Dilate Steps', ...
                                                'Callback', @(src, evt)callbackButtDilate(obj));
            asm_boxes.butts.edit_dilate = uicontrol('Parent', asm_boxes.butt_d_boxes, ...
                                                'Style', 'edit', 'String', '1');

            obj.conf.assemble_boxes = asm_boxes;
        end

        function resetUiComponents(obj)
            params = fieldnames(obj.conf.assemble_boxes.params);
            for ii = 1:length(params)
                param = params{ii};
                value = obj.toString(obj.assembler.localPar.(param));
                set(obj.conf.assemble_boxes.params.(param).edit, 'String', value);
            end
        end

        function doUpdateDisplay(obj)
        end

        function addUICallbacks(obj)
        end

        function conf = collectConf(obj)
            conf = {};
            params = fieldnames(obj.conf.assemble_boxes.params);
            for ii = 1:length(params)
                param = params{ii};
                value = get(obj.conf.assemble_boxes.params.(param).edit, 'String');
                conf(end+1:end+2) = {param, obj.fromString(param, value)};
            end
        end

        function asm_boxes = createConfChoices(obj, asm_boxes)
            params = fieldnames(obj.defaultConf);
            for ii = 1:length(params)
                param = params{ii};
                if (ismember(param, {'autosave', 'cache', 'update_status_cb', ...
                                     'add_to_filename', 'list'}))
                    continue;
                end
                asm_boxes.params.(param) = [];
                asm_boxes.params.(param).boxes = uiextras.HBox('Parent', asm_boxes.param_boxes);
                asm_boxes.params.(param).label = uicontrol('Parent', asm_boxes.params.(param).boxes, ...
                                                        'Style', 'text', ...
                                                        'String', param, ...
                                                        'HorizontalAlignment', 'left');
                asm_boxes.params.(param).edit = uicontrol('Parent', asm_boxes.params.(param).boxes, ...
                                                        'Style', 'edit');
                set(asm_boxes.params.(param).boxes, 'Sizes', [-1 60]);
            end

            sizesVec = ones(1, length(asm_boxes.param_boxes.Children)) * 30;
            set(asm_boxes.param_boxes, 'Sizes', sizesVec);
        end

        function value = fromString(obj, param, value)
            content = obj.defaultConf.(param);
            if (islogical(content))
                if (strcmpi(value, 'true'))
                    value = true;
                elseif (strcmpi(value, 'false'))
                    value = false;
                else
                    error('GUI_ASSEMBLE:wrong_argument', ...
                          'Unknown logical "%s"', value);
                end
            elseif (isnumeric(content))
                if (isnumeric(value))
                    value = str2double(value);
                end
            end
            if (isnan(value))
                value = '';
            end
        end
    end

    methods (Access = public)
        function callbackButtSinglePhase(obj)
            conf = obj.collectConf();

            obj.assembler.changeParams(conf);

            value = get(obj.conf.assemble_boxes.butts.edit_phase, 'String');
            value = str2double(value);

            obj.assembler.assemblePhase(value);
        end

        function callbackButtAllPhases(obj)
            conf = obj.collectConf();

            obj.assembler.changeParams(conf);

            obj.assembler.assembleAllPhases();
        end

        function callbackButtVolume(obj)
            conf = obj.collectConf();

            obj.assembler.changeParams(conf);

            obj.assembler.assembleCompleteVolume();
        end

        function callbackButtDilate(obj)
            conf = obj.collectConf();

            obj.assembler.changeParams(conf);

            value = get(obj.conf.assemble_boxes.butts.edit_dilate, 'String');
            value = str2double(value);

            obj.assembler.dilateCompleteVolume(value);
        end

        function callbackButtReset(obj)
            obj.resetUiComponents();
        end
    end
end
