function output = gtFindSimilarGrains(g1, g2, parameters1, parameters2)
%find similar grains
%given the grain structure generated by sort_grains, containing positions
%and R-vectors, and the parameters files, containing pixel size data...

% Extract all the Rodrigues vectors into row vectors
map1.R_vec = gtIndexAllGrainValues(g1, 'R_vector', [], 1, 1:3);
map2.R_vec = gtIndexAllGrainValues(g2, 'R_vector', [], 1, 1:3);

% Extract all the grain centers into row vectors
map1.pos = gtIndexAllGrainValues(g1, 'center', [], 1, 1:3);
map2.pos = gtIndexAllGrainValues(g2, 'center', [], 1, 1:3);

% Get symmetry operators
symm = gtCrystGetSymmetryOperators(parameters1.cryst.crystal_system, ...
                                   parameters1.cryst.spacegroup); 

%try to deal with shifts here.
map1.pos=map1.pos-repmat(mean(map1.pos,1), length(map1.pos), 1);
map2.pos=map2.pos-repmat(mean(map2.pos,1), length(map2.pos), 1);
%scale to pixelsize of map 1
map2.pos=map2.pos*parameters2.acq.pixelsize/parameters1.acq.pixelsize;

% Compute all orientation matrices g on each map
all_g1 = gtMathsRod2OriMat(map1.R_vec.');
all_g2 = gtMathsRod2OriMat(map2.R_vec.');

% Go through grain map 1, looking for matches in grain map 2
output = [];
for ii = 1:size(all_g1, 3)
    g1 = all_g1(:, :, ii);

    % Calculate misorientations
    % TODO: replace with a MISORIENTATION function
    misorientation = [];
    for jj = 1:size(all_g2, 3)
        g2 = all_g2(:, :, jj);
        % Need to search symmetry equivalents for the minimum misorientation
        rot_offset=[];
        for k = 1:length(symm)
            g1equiv = symm(k).g3*g1;
            netg = inv(g1equiv)*g2;
            rot_offset(k) = acos((trace(netg)-1)/2);
        end
        misorientation(jj) = min(abs(rot_offset));
    end

    % Calculate distances
    dif = map2.pos-repmat(map1.pos(ii,:),size(map2.pos,1),1);
    distance = sqrt(sum((dif.^2),2));

    pos=find(distance<70 & misorientation'<5*pi/180);
    disp([ii pos]);
    output(ii).possibles=pos;
    output(ii).dif=dif(pos,:);
    output(ii).distance=distance(pos);

    output(ii).misorientations=misorientation(pos);

end

keyboard

end % end of function
