function hkil = gtCart2Hex(hkl, latticepar)
% GTCART2HEX  Convert a plane normal in cartesian coordinates (hkl) to the four
% indexes
%
%     hkil = gtCart2Hex(hkl, latticepar)
%     ----------------------------------
%     INPUT:
%       hkl  = plane normal Miller indexes <double Nx3>
%       latticepar = lattice parameters <double 1x6>
%
%     OUTPUT:
%       hkil = cartesian coordinates for hexagonal materials
%
%     Version 002 25-11-2011 by LNervo
%       *** update version ***
%
%     Version 001 June-2008 by AKing
%       Four index hexagonal coordinates are given!
%       reverse of the formulae implemented by Sabine in gtForwardSimulate_360
%       the output {hkil} is not normalised in any way.


hkil=zeros(size(hkl,1), 4);

%change to hexagonal axes
hkil(:,1) = hkl(:,1) - (hkl(:,2)/sqrt(3));
hkil(:,2) = hkl(:,2)*2/sqrt(3);
hkil(:,3) = -(hkil(:,1)+hkil(:,2));
hkil(:,4) = hkl(:,3);

%allow for unit cell parameters
hkil(:,1:3) = hkil(:,1:3)*(latticepar(1)*sqrt(3))/2;
hkil(:,4)   = hkil(:,4)*latticepar(3);

end % end function
