function [symm, opsym] = gtCrystCalculateSymmetryOperators(sg, crystal_system)
% GTCRYSTCALCULATEREFLECTIONS  Calculate the symmetry operators list using python 
%                              from xfab library in fable (sg.py)
%     [symm, opsym] = gtCrystCalculateSymmetryOperators(sg, crystal_system)
%     ---------------------------------------------------------------------
%
%     INPUT: 
%       sg             = spacegroup number <int>
%       crystal_system = crystal system <string>
%     
%     OUTPUT:
%       symm = <struct 1xN> contains :
%           .perm = set of indistinguishable lattice permutations <double 3x3>
%           .g3   = set of unitary rotation matrices corresponding to the
%                   indistinguishable lattice permutations <double 3x3>
%       opsym = <struct 1xM> contains :
%            .sg_rot   = rotations (complete list using symmetry operators) <double 3x3>
%            .sg_trans = transition values from symmetry operators <double 1x3>
%
%
%     Version 001 10-12-2012 by LNervo


if ~exist('sg','var') || isempty(sg)
    load parameters;
    sg = parameters.cryst(1).spacegroup;
end
if ~exist('crystal_system','var') || isempty(crystal_system)
    load parameters;
    crystal_system = parameters.cryst(1).crystal_system;
end

global GT_MATLAB_HOME;
script_file = fullfile(GT_MATLAB_HOME, 'zUtil_Python', 'symmetry_operators_list.py');
[~, msg] = gtPythonCommand([script_file ' ' num2str(sg)], true);
disp(msg);

% read produced files      
permN     = gtReadTextFile(['perm_' crystal_system '.txt'],'%f %f %f',' ','#',[3 3]);
rotN      = gtReadTextFile(['rot_' crystal_system '.txt'],'%f %f %f',' ','#',[3 3]);
sg_rotN   = gtReadTextFile(['sg_rot_' crystal_system '.txt'],'%f %f %f',' ','#',[3 3]);
sg_transN = gtReadTextFile(['sg_trans_' crystal_system '.txt'],'%f %f %f',' ','#',[1 3]);

symm = struct('g3', rotN, 'perm', permN);
if nargout == 2
    opsym = struct('sg_rot', sg_rotN, 'sg_trans', sg_transN);
end

end % end of function
