/*
 * internal_cell_dot_product.cpp
 *
 *  Created on: Jul 13, 2012
 *      Author: ben
 */

#include "internal_cell_defs.h"

const char * errorID = "C_FUN:cell_dot_product:wrong_argument";

void mexFunction( int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[] )
{
  if (nrhs != 2) {
    mexErrMsgIdAndTxt(errorID,
        "Not enough arguments! you need to provide two cell arrays");
  }
  if (!(mxIsCell(prhs[0]) && mxIsCell(prhs[1]))) {
    mexErrMsgIdAndTxt(errorID,
        "The arguments need to be Cell arrays");
  }

  const mxClassID firstClass = getMxClassOfCell(prhs[0]);
  if (firstClass != getMxClassOfCell(prhs[1])) {
    mexErrMsgIdAndTxt(errorID,
        "The arguments need to be Cell arrays of coherent floating point types");
  }

  switch (firstClass) {
    case mxDOUBLE_CLASS: {
      plhs[0] = mxCreateDoubleScalar(cell_dot_product_sse<double>(prhs[0], prhs[1]));
      break;
    }
    case mxSINGLE_CLASS: {
      plhs[0] = mxCreateNumericMatrix(1, 1, mxSINGLE_CLASS, mxREAL);
      float * const elem = (float *) mxGetData(plhs[0]);
      *elem = cell_dot_product_sse<float>(prhs[0], prhs[1]);
      break;
    }
    case mxUNKNOWN_CLASS: {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a non empty Cell array of coherent floating point types");
    }
    default: {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a Cell array of floating point numbers");
    }
  }
}



