/*
 * internal_cell_square_norm.cpp
 *
 *  Created on: Jul 13, 2012
 *      Author: ben
 */

#include "internal_cell_defs.h"

const char * errorID = "C_FUN:cell_square_norm:wrong_argument";

void mexFunction( int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[] )
{
  if (nrhs != 1) {
    mexErrMsgIdAndTxt(errorID,
        "Not enough arguments! you need to provide a cell array");
  }
  if (!mxIsCell(prhs[0])) {
    mexErrMsgIdAndTxt(errorID,
        "The argument needs to be a Cell array");
  }

  switch (getMxClassOfCell(prhs[0])) {
    case mxDOUBLE_CLASS: {
      plhs[0] = mxCreateDoubleScalar(cell_square_norm_sse<double>(prhs[0]));
      break;
    }
    case mxSINGLE_CLASS: {
      plhs[0] = mxCreateNumericMatrix(1, 1, mxSINGLE_CLASS, mxREAL);
      float * const elem = (float *) mxGetData(plhs[0]);
      *elem = cell_square_norm_sse<float>(prhs[0]);
      break;
    }
    case mxUNKNOWN_CLASS: {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a non empty Cell array, of coherent floating point numbers");
    }
    default: {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a Cell array of floating point numbers");
    }
  }
}



