function grain = gtIndexEquivalentPairs(grain)
% GTINDEXEQUIVALENTPAIRS Finds the equivalent Friedel pairs (two pairs
% that originate from the same plane) in all grains.
%
% grain = gtIndexEquivalentPairs(grain)
%
% ------------------------------------------------------------
%
% INPUT
%   grain - all grain data as in index.mat
%
% OUTPUT
%   grain.equipair - linear indices of the corresponding Friedel pairs;
%                    zero if no corresponding pair was found.
%   grain.equilist - list of equivalent Friedel pair indices


for ii = 1:length(grain)

    grain{ii}.equipair = zeros(1,length(grain{ii}.hklind));
    grain{ii}.equilist = [];
    
    % Loop through Friedel pairs of the grain
    for jj = 1:length(grain{ii}.hklind);
        
        % Check which linear indices of {hkl} and (hkl) match
        inds1     = (grain{ii}.hklind  == grain{ii}.hklind(jj));
        inds2     = (grain{ii}.shklind == grain{ii}.shklind(jj));
        inds2(jj) = false; 

        % Index of equivalent pair
        equiind = find(inds1 & inds2);
        
        if ~isempty(equiind)   
            
            % Maximum one equivalent pair can exist
            if (length(equiind) > 1)
                msg = sprintf('More than two equivalent Friedel pairs found for pair #%d in grain #%d', jj, ii);
                warning(msg)
                disp('The equivalent pair with the lower index was selected.')
            end
            
            grain{ii}.equipair(jj) = equiind(1);
            
            if (jj < grain{ii}.equipair(jj))
                grain{ii}.equilist(end+1,:) = [jj grain{ii}.equipair(jj)];
            end
        end
        
    end
    
end


end % of function