function gtSetupIndexing()
% GTSETUPINDEXING  Helper function to set the parameters for Indexter.
%     gtSetupIndexing()
%     -----------------

parameters = [];
load('parameters.mat');

% Let's check every field is in, and in case let's update it!
parameters.('index') = gtCheckParameters(parameters.('index'), gtIndexDefaultParameters());
parameters.('index').('strategy') = gtCheckParameters(parameters.('index').('strategy'), gtIndexDefaultStrategy());

save('parameters.mat', 'parameters');

% Use default parameters or review/edit them?
check = inputwdefault('Use existing parameters for phase #1 for gtINDEXTER? [y/n]', 'y');

if strcmpi(check, 'y')

    % Index phase 01 by default
    fname = ('4_grains/phase_01/index_input.mat');

    if exist(fname,'file')
        disp('The existing input file for phase #1 will be used.')
    else
        disp('An input file for phase #1 will be created.')
    end

    check2 = inputwdefault('Start gtINDEXTER now? [y/n]', 'y');
    if strcmpi(check2, 'y')

        if exist(fname,'file')
            gtINDEXTER(1, 1, 1); % use input file
        else
            gtINDEXTER(1, 1, 0); % do not use input file
        end

    end

else

    % Work through the various INDEXTER parameters

    % Get parameters template
    list = build_list_v2;

    % Select phase to be indexed
    header = 'INDEXTER OPTIONS';

    options.phase_ID = 1;

    oplist{1,1} = 'phase_ID';
    oplist{1,2} = 'Which phase should be indexed?';

    options = gtModifyStructure(options, oplist, header);


    % Update and log
    options.flag_update = true;
    options.flag_log    = true;

    % Input file name
    fname = sprintf('4_grains/phase_%02d/index_input.mat',options.phase_ID);

    if exist(fname,'file')
        header = 'An input file already exists.';
        options.input_file = true;
        oplist{1,1} = 'input_file';
        oplist{1,2} = 'Use input file?';
        oplist{2,1} = 'flag_update';
        oplist{2,2} = 'Update ''parameters.mat'', ''index.mat'', database with the results ?';
        oplist{3,1} = 'flag_log';
        oplist{3,2} = 'Write log and output file with current date ?';
        options = gtModifyStructure(options, oplist, header);
    else
        oplist{1,1} = 'flag_update';
        oplist{2,2} = 'Update ''parameters.mat'', ''index.mat'', database with the results ?';
        oplist{2,1} = 'flag_log';
        oplist{2,2} = 'Write log and output file with current date ?';
        options = gtModifyStructure(options, oplist);
        options.input_file = false;
    end


    % Initial parameters
    header = 'General parameters';

    par.iter   = parameters.index.strategy.iter;
    par.rfzext = parameters.index.strategy.rfzext;

    par = gtModifyStructure(par, list.index__strategy(1:2,1:2), header);

    parameters.index.strategy.iter   = par.iter;
    parameters.index.strategy.rfzext = par.rfzext;


    % Tolerances for building grains in the first iteration:
    header = 'Tolerances for BUILDING grains in the FIRST iteration loop';
    parameters.index.strategy.b.beg = gtModifyStructure(parameters.index.strategy.b.beg,...
        list.index__strategy__b__beg(:,1:2), header);


    % Tolerances for building grains in the last iteration:
    header = 'Tolerances for BUILDING grains in the LAST iteration loop';
    parameters.index.strategy.b.end = gtModifyStructure(parameters.index.strategy.b.end,...
        list.index__strategy__b__end(:,1:2), header);


    % Tolerances for merging grains in the first iteration:
    header = 'Tolerances for MERGING grains in the FIRST iteration loop';
    parameters.index.strategy.m.beg = gtModifyStructure(parameters.index.strategy.m.beg,...
        list.index__strategy__m__beg(:,1:2), header);


    % Tolerances for merging grains in the last iteration:
    header = 'Tolerances for MERGING grains in the LAST iteration loop';
    parameters.index.strategy.m.end = gtModifyStructure(parameters.index.strategy.m.end,...
        list.index__strategy__m__end(:,1:2), header);


    % Tolerances for adding unindexed pairs to grains based on statistics.
    header = 'Tolerances for ADDING unindexed pairs to grains based on statistics';
    parameters.index.strategy.s = gtModifyStructure(parameters.index.strategy.s,...
        list.index__strategy__s(:,1:2), header);


    % Tolerances for excluding indexed pairs from grains based on statistics.
    header = 'Tolerances for EXCLUDING indexed pairs from grains based on statistics';
    parameters.index.strategy.x = gtModifyStructure(parameters.index.strategy.x,...
        list.index__strategy__x(:,1:2), header);


    if options.flag_update
        disp('Saving updated parameters into parameters file...')
        save('parameters.mat','parameters');

        check2 = inputwdefault('Start gtINDEXTER now? [y/n]', 'y');
        if strcmpi(check2, 'y')
            gtINDEXTER(options.phase_ID, options.flag_update, ...
                options.input_file, options.flag_log);
        end

    else
        disp('Indexing parameters will not be saved...')

        check2 = inputwdefault('Start gtINDEXTER now? [y/n]', 'y');
        if strcmpi(check2, 'y')
            gtINDEXTER(options.phase_ID, options.flag_update, ...
                options.input_file, options.flag_log, 0, parameters);
        end

    end


end
