function compiled_fine = gt_mcc(myfunctionname, varargin)
% gt_mcc.m Compiles an executable and puts it in the right place
    if (isempty(whos('global', 'GT_MATLAB_HOME')))
        error('COMPILE:no_such_variable', ...
              ['GT_MATLAB_HOME variable doesn''t exist in global workspace, '...
               'your environment is not sane, and you need to either ' ...
               're-initialise or re-run matlab.'])
    end
    global GT_MATLAB_HOME
    try
        v = ver('compiler');
        mcc_vers = textscan(v.Version, '%s','delimiter','.');
        mcc_vers_minor = str2double(mcc_vers{1}{2});
        mcc_vers_major = str2double(mcc_vers{1}{1});
        compiled_fine = true; % In case of error, we will re modify it
    catch mexc
        gtPrintException(mexc, 'Error: Is the Matlab Compiler installed?')
        compiled_fine = false;
        return
    end

    if ( mcc_vers_major < 4 || (mcc_vers_major == 4 && mcc_vers_minor < 8))
        disp('ERROR: Compilation using Matlab versions before R2008 is not supported')
        compiled_fine = false;
        return
    end

    % Parsing arguments
    conf = [];
    conf.not_really = false;
    conf.out_path = '';

    conf = parse_pv_pairs(conf, varargin);

    if (isempty(conf.out_path))
        path_for_binaries = fullfile(GT_MATLAB_HOME, 'bin', 'compiled');
    else
        path_for_binaries = conf.out_path;
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Modify the M name if necessary
    if ~strcmp(myfunctionname(end-1:end), '.m')
        myfunctionname = [myfunctionname '.m'];
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % check the code is visible at the moment
    if ~exist(myfunctionname, 'file')
        disp('Cannot find your function - is it in the path?')
        compiled_fine = false;
        return
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Adding Global Workspace
    globalVars = whos('global');
    eval(['global' sprintf(' %s', globalVars.('name'))]);
    if (isempty(whos('global', 'GT_DB')))
        error('COMPILE:no_such_variable', ...
              ['GT_DB variable doesn''t exist in global workspace, '...
               'your environment is not sane, and you need to either ' ...
               're-initialise or re-run matlab.'])
    end
    save('workspaceGlobal.mat', 'GT_DB', 'GT_MATLAB_HOME');

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % startup the compiler
    disp('Starting to compile - this may take a few minutes...');
    disp('- Removing previously compiled code');
    compFile = fullfile(path_for_binaries, [myfunctionname(1:end-2)]);
    runFile = fullfile(path_for_binaries, ['run_' myfunctionname(1:end-1) 'sh']);
    filesToDelele = {compFile, runFile};
    for fileToDelele = filesToDelele
        if (conf.not_really)
            disp(['I should now be deleting: ' fileToDelele{1}]);
        elseif exist(fileToDelele{1}, 'file')
            delete(fileToDelele{1});
        end
    end

    disp(['- Placing result in ' path_for_binaries]);
    if (conf.not_really)
        cmd = sprintf('mcc -m "%s" -d "%s" -a workspaceGlobal.mat', myfunctionname, path_for_binaries);
        disp(['I should now be executing: ''' cmd '''']);
    else
        try
            mcc('-m', myfunctionname, '-d', path_for_binaries, '-a', 'workspaceGlobal.mat');
        catch mexc
            if gtCheckExceptionType(mexc, 'mcc_err_checkout_failed')
                disp('Could not get a compiler license.');
                disp('Surf to http://compweb/public/nice/matlab_licence.php');
                disp('and call the person with the compiler license to see if you could use it!');
            else
                gtPrintException(mexc);
            end
            compiled_fine = false;
            return
        end
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Clean the folder
    delete('workspaceGlobal.mat');

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % And let the user know everything is finished
    disp('PLEASE RESTART MATLAB TO FREE THE COMPILER FOR SOMEONE ELSE!');
end
