'''
Created on May 2, 2013

@author: ben
'''

import re
import fnmatch
import os

import dct_io_xml

class DCTBatchConf(dct_io_xml.DCTXMLBase):

    @staticmethod
    def getNewFile(xml_file):
        tree = dct_io_xml.ET.Element("batch")
        etree = dct_io_xml.ET.ElementTree(tree)
        etree.write(xml_file, pretty_print = True)
        conf = DCTBatchConf(xml_file, read_only = False)
        conf.fixStructure()
        return conf

    def __init__(self, xml_file = os.path.join("zUtil_Conf", "batch-dev-conf.xml"), read_only = True):
        dct_io_xml.DCTXMLBase.__init__(self, xml_file)
        self.read_only = read_only

    def fixStructure(self):
        if self.tree.find("options") is None:
            dct_io_xml.ET.SubElement(self.tree.getroot(), "options")
        if self.tree.find("functions") is None:
            dct_io_xml.ET.SubElement(self.tree.getroot(), "functions")

    def getPath(self, xml_path, base_dir, base_node = None):
        if base_node is None:
            node = self.tree.find(xml_path)
        else:
            node = base_node.find(xml_path)
        if node is not None:
            if node.attrib.get("absolute") == "false":
                return os.path.join(base_dir, node.text)
            else:
                return node.text
        else:
            raise ValueError("No path defined for '%s" % xml_path)

    def getBinDir(self, base_dir):
        return self.getPath("options/binaries/path", base_dir)

    def addFunction(self, name, path, is_absolute = False):
        if self.read_only is True:
            raise ValueError("File %s is open in read only mode" % self.xmlFile)

        func = self.tree.find("functions/function[@name='%s']" % name)
        if func is None:
            funcs = self.tree.find("functions")
            func = dct_io_xml.ET.SubElement(funcs, "function", {"name" : name})

        pathNode = func.find("path")
        if pathNode is None:
            pathNode = dct_io_xml.ET.SubElement(func, "path")

        pathNode.text = path
        if is_absolute is True:
            pathNode.attrib["absolute"] = "true"
        else:
            pathNode.attrib["absolute"] = "false"

        self.save()

    def getFunctions(self, base_dir):
        functions = {}
        nodes = self.tree.findall("functions/function")
        for node in nodes:
            functions[node.attrib.get("name")] = self.getPath("path", base_dir, base_node = node)
        return functions

class DCTBatch(object):

    def __init__(self, dct_dir = "", dev_mode = False):
        if dct_dir is "":
            dct_dir = os.getcwd()
        self.dct_dir = dct_dir

        devel_conf_path = os.path.join(self.dct_dir, "zUtil_Conf", "batch-dev-conf.xml");
        try:
            self.dev_conf = DCTBatchConf(xml_file = devel_conf_path, read_only = (not dev_mode) )
        except:
            self.dev_conf = DCTBatchConf.getNewFile(devel_conf_path)

        user_conf_path = os.path.join(self.dct_dir, "batch-user-conf.xml");
        try:
            self.user_conf = DCTBatchConf(xml_file = user_conf_path, read_only = False)
            # We assume system configuration is always consistent, so we only
            # check user configuration
            self.user_conf.fixStructure()
        except:
            self.user_conf = DCTBatchConf.getNewFile(user_conf_path)

    def autoDetectFunctions(self, user = True):
        simpleMatch = 'OAR_make'
        searchPattern = re.compile('[^%]*\s*' + simpleMatch + "\s*\(\s*'.*'")

        filesList = []
        for root, dir_name, file_names in os.walk(self.dct_dir):
            for file_name in file_names:
                if fnmatch.fnmatch(file_name, '*.m'):
                    filesList.append((root, file_name))

        # First creating list of file matching methods name
        mFiles = {}
        for dir_path, file_name in filesList:
            filepath = os.path.join(dir_path, file_name)
            mFiles[file_name[:-2]] = filepath

        # Then looking for complex regexp inside matching files
        for dir_path, file_name in mFiles.iteritems():
            filepath = os.path.join(dir_path, file_name)
            fid = open(filepath, 'r')

            for line in fid:
                if simpleMatch in line:
                    matched = searchPattern.search(line)
                    if matched:
                        function_name = matched.group().split("'")[1]
                        self.addFunction(mFiles[function_name], user)

            fid.close()

    def addFunction(self, path, user = True):
        name = os.path.basename(path)
        name = name[0:-2]

        # Let's make the path relative
        abs_path = os.path.abspath(path)
        abs_dct_path = os.path.abspath(self.dct_dir)
        rel_path = abs_path[len(abs_dct_path)+1:]

        if user is True:
            self.user_conf.addFunction(name, rel_path)
        else:
            self.dev_conf.addFunction(name, rel_path)

    def getBinDir(self):
        try:
            bin_path = self.user_conf.getBinDir(self.dct_dir)
        except ValueError:
            bin_path = self.dev_conf.getBinDir(self.dct_dir)
        return bin_path

    def getFunctions(self):
        functions = self.dev_conf.getFunctions(self.dct_dir)
        functions.update(self.user_conf.getFunctions(self.dct_dir))
        return functions

if __name__ == '__main__':
    pass

